#!/bin/bash

set -ue

source ./config.ini

function parse_json() {
    local _result=$1
    local _json_key=$2
    echo "${_result//\"/}" | sed "s/.*$_json_key:\([^,}]*\).*/\1/"
}

function check_middleware_proxy() {
    if ! /usr/sbin/ss -lnpt | grep 30003 &> /dev/null;then
        log error  "middleware proxy需要开启，通过proxy访问容器ES实例，请在compose中调整 replicas为1 ..."
        exit 1000
    fi
    log info "middleware proxy已开启, 监听端口30003"
}

function check_es_status() {
    if curl -s $ES_URL/_cat/health | grep green &> /dev/null;then
        log info "Elasticsearch status is green, check OK ..."
    elif curl -s $ES_URL/_cat/health | grep red &> /dev/null;then
        log error "Elasticsearch status check is red, recovery 异常 ..."
        exit 1001
    elif curl -s $ES_URL/_cat/health | grep yellow &> /dev/null;then
        # local result=$(curl -s $ES_URL/_cat/health | awk '$NF != "50.0%" {print $0}')
        # if [ ! -z "$result" ];then
        #     log error "Elasticsearch 单机部署模式，状态异常，请进行检查，result: $result ..."
        # fi
        log info "Elasticsearch status yellow, check OK ..."
    fi
}

function create_snapshot_warehouse() {
    if [ $(curl -sIL -w "%{http_code}\n" -o /dev/null --location --request GET "$ES_URL/_snapshot/macrowing") != 200 ];then
    	log info "Elasticsearch snapshots warehouse macrowing not exist, create it ... "
        curl -L -X POST "$ES_URL/_snapshot/macrowing" -H 'Content-Type: application/json' -d '
        {
            "type": "fs",
            "settings": {
                "location": "edoc2_backup"
            }
        }
        ' &> /dev/null
    fi
}

function check_backup_status() {
    log info "Start check es backup file..."
    local stat=$(curl -sL -X GET "${ES_URL}/_snapshot/macrowing/${1}")
    if [ "$(parse_json $stat "state")" == "SUCCESS" ]; then
        log info "Elasticsearch macrowing snapshot ${1} is success."
    else
        log error "Elasticsearch macrowing snapshot ${1} is failed. Please Check!"
        exit 1 ;
    fi
}

function create_snapshot() {
    log info "Start elasticsearch backup..."
    create_snapshot_warehouse
    curl -sL -X PUT "${ES_URL}/_snapshot/macrowing/${1}?wait_for_completion=true" &> /dev/null
    check_backup_status ${1}
}

function get_es_dir() {
    es_back_dir=$(docker inspect $(docker ps | grep elasticsearch | awk '{print $1}') --format '{{range .Mounts}}{{if eq .Destination "/esbackup"}}{{.Source}}{{end}}{{end}}')
    # if [ -n "$es_back_dir" ];then
    #     local _ret=$1
    #     eval $_ret='$es_back_dir'
    # else
    #     log_error "获取宿主机ES备份目录失败，请检查执行脚本机器是否运行ES容器。。"
    #     exit
    # fi
    es_node=$(curl -sL -X GET "${ES_URL}/_nodes/stats?pretty" | head -n 3  | grep total| awk  '{print $3}' |sed 's/,//')
    if [ -z "$es_back_dir" -o $es_node -eq 3 ];then
        # 检查是否是v6.7.1.11版本以上，未挂载 /esbackup 目录情况，需要报错退出
        check_esbackup=$(docker exec $(docker ps | grep elasticsearch |  awk '{print $1}')  [ -d /esbackup ] && echo "true")
        if [ "$check_esbackup" = "true" ];then
            es_back_dir=$init_es_datadir/backup
        else
            log error "未挂载 es 备份目录 /esbackup,请在 compose 中挂载 /esbackup 目录后继续执行"
            exit 11;
        fi
    else
        es_back_dir=$es_back_dir/edoc2_backup
    fi
}

function copy_es_backup_to_share() {
    if [ ${init_server-"false"} = "true" ]; then
        get_es_dir
        [ ! -d $share_tmp_dir ] && mkdir $share_tmp_dir
        # cp -rp $init_es_datadir/backup $share_tmp_dir/edoc2_backup/
        cp -rp $es_back_dir $share_tmp_dir/edoc2_backup/
        if [ $? -eq 0 ];then
            log info "ES copy backup ${es_back_dir}/backup to ${share_tmp_dir}/es success."
        else
            log error "ES copy backup ${es_back_dir}/backup to ${share_tmp_dir}/es failed."
            exit 1;
        fi
    else
        [ ! -d $share_tmp_dir/es ] && log error "Please check localhost isn't init_server." && exit 1;
    fi
}

function copy_backup_to_local() {
    get_es_dir
    if [ ! -d $cluster_es_datadir ]; then
        log error "$cluster_es_datadir doesn't exist."
        exit 1;
    fi

    # if [ "$init_es_datadir" != "$cluster_es_datadir" -a "$(ls -A $cluster_es_datadir)" != "" ]; then
    #     log error "$cluster_es_datadir is not empty!"
    #     exit 1 ;
    # fi
    log info "Copy es backup to local."
    check_esbackup=$(docker exec $(docker ps | grep elasticsearch |  awk '{print $1}')  [ -d /esbackup ] && echo "true")
    if [ "$check_esbackup" = "true" ];then
        docker cp -a -L $share_tmp_dir/edoc2_backup $(docker ps | grep elasticsearch |  awk '{print $1}'):/esbackup
    else
        cp -rp $share_tmp_dir/edoc2_backup/* $es_back_dir
    fi
}

function close_index(){
    log info "close ${ES_URL} index _all "
    while true; do
        local resp_code=$(curl -sIL -w "%{http_code}\n" -o /dev/null -X POST "${ES_URL}/_all/_close")
        if [ "$resp_code"x == "200"x ];then
            log info "close $ES_URL index _all is SUCCESS "
            break
        elif [ "$resp_code"x == "403"x ];then
            curl -XPUT -H "Content-Type: application/json" "${ES_URL}/_all/_settings" -d '{"index.blocks.read_only_allow_delete": null}' &> /dev/null
            continue
        else
            log_error "close $ES_URL index is FAILED "
            exit 2003
        fi
    done
}

function check_recovery_status() {
    log info "Check recovery status..."
    local indices=( $(curl -s ${ES_URL}/_cat/indices | awk '{print $3}' | xargs) )
    for i in  ${!indices[@]}
    do  
        sleep 1
        snap_num=$(curl -s ${ES_URL}/${indices[$i]}/_recovery | grep -o  '"type":' | wc -l)
        snap_done_num=$(curl -s ${ES_URL}/${indices[$i]}/_recovery | grep -o  '"stage":"DONE"' |wc -l)
        if [ "$snap_num" != "$snap_done_num" ];then
            log error "${indices[$i]} index recovery not finished."
        else
            log info "${indices[$i]} index recovery commplate."
        fi
    done
}

function recovery_from_snapshot() {
    check_backup_status $SNAPSHOT_NAME
    close_index
    curl -s -X POST "${ES_URL}/_snapshot/macrowing/${SNAPSHOT_NAME}/_restore"
    if [ $? -eq 0 ];then
        log info "es recovery commplate."
    else
        log error "es recovery failed!"
        exit 1;
    fi
    check_recovery_status
}


# SNAPSHOT_NAME=snapshot_all_indices_$(date +'%Y%m%d%H%M%S')
SNAPSHOT_NAME=snapshot_all_indices_backup


function backup() {
    check_middleware_proxy
    # backup
    check_es_status
    create_snapshot $SNAPSHOT_NAME
    copy_es_backup_to_share
}

function get_backup() {
    check_es_status
    create_snapshot_warehouse
    copy_backup_to_local
}

function recovery() {
    check_middleware_proxy
    # recovery
    # create_snapshot_warehouse
    # copy_backup_to_local
    recovery_from_snapshot
    check_es_status
}

${1}
